#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

//----------------------------------------------------------------------------------
// Read in and do some simple checks on the input parameters. If parameter j is
// messed up, this returns -j as the error flag. I have hard-coded in some limits:
// 3 < N < 20000, maxits > 0, and tolerance >= 1.0e-16. Change them if you really 
// want to. But you really don't want to.
//
// The verbose variable should be kept to 0.
//
// The variable "method" is intended to take on some value in the set {1..4}:
//  1. Use Jacobi method, no OpenMP
//  2. Use Jacobi method, with OpenMP
//  3. Use Gauss-Seidel method, no OpenMP
//  4. Use Gauss-Seidel method, with OpenMP
//
//----------------
// Randall Bramley
// Department of Computer Science
// Indiana University, Bloomington
//----------------
// Last Modified: Wed 24 Jan 2018, 01:11 PM
//----------------------------------------------------------------------------------
int getparms(int *N, int *maxits, double *tolerance, int *printfreq, int*showplate, 
        int*method) {

    int talkalot = 0;   // Verbosity control
    FILE *parameters;   // input file
    int howmany;        // error from fprintf

    parameters = fopen("parameters", "r");
    if (parameters == NULL) {
        printf("Could not open required input file for parameters\n");
        return(-2);
    }
    howmany = fscanf(parameters, "%d %d %lf %d %d %d", 
            N, maxits, tolerance, printfreq, showplate, method);
    fclose(parameters);

    // Basic sanity tests. printfreq is allowed to be non-positive, to
    // indicate that the convergence history is not to be tracked.
    if (*N < 2){
        // N = 1 implies just one unknown point. The code may work, but why would
        // you do that? Perversity?
        printf("getparms: N is too small! N = %d\n", *N);
        return -1;
    }

    if (*N > 20000){
        // The problem size is O(N^2), and in fact will require over 16*N^2 bytes.
        // For N = 20000, that will be 6.4 Gbytes. You can bump that up if you
        // really want to get much larger problem sizes, but don't do it on any of
        // *my* machines. And stay off my lawn, also.
        printf("getparms: N is too big. N = %d\n", *N);
        return -1;
    }

    if (*maxits < 1){
        // The number of iterations required will likely be vastly larger than N.
        // You may want to make it smaller for debugging or other testing. However,
        // this thing needs to make at least one iteration.
        printf("getparms: maxits is too small! maxits = %d\n", *maxits);
        return -2;
    }

    if (*tolerance < 1.0e-16){
        // Smaller may be OK, but will lead to iterating forever, or at least
        // until maxits is hit. So I'm flagging it as an error. If you really
        // want to drive this further, change the parameter 1.0e-16. Hint:
        // you really don't really want to do that.
        printf("getparms: tolerance is too small! tolerance = %e\n", *tolerance);
        return -3;
    }

    if (*method < 1 || *method > 4){
        // If you add other methods, may need to change the test here.
        printf("getparms: Unknown method = %d\n", *method);
        return -4;
    }

    if (talkalot) {
        printf("getparms: N = %d\n", *N);
        printf("getparms: maxits = %d\n", *maxits);
        printf("getparms: tolerance = %e\n", *tolerance);
        printf("getparms: printfreq = %d\n", *printfreq);
        printf("getparms: showplate = %d\n", *showplate);
    }

    return 0;
}
