!===========================================================================
! This version of function elapsedtime.f90 can be used directly 
! because it does not have an optional input parameter.
!-----------------------------------------------------------------------
! 
! The elapsedtime.f90 function returns a double precision number,
! the number of seconds since some unspecified but specific event.
! Use the "which_version" parameter to choose among the variants,
! or trust the default system_clock_version setting here. The possible
! values are:
!
!    which_version = 1 for date_and_time()
!    which_version = 3 for cpu_time()
!    which_version = 4 for system_clock() with 4-byte integers
!    which_version = 8 for system_clock() with 8-byte integers
!
! Note 1: the system_clock() version doesn't check against overflow
! in the counter - add the argument count_max for that if you need
! it. date_and_time() is more robust, but won't work across monthly 
! boundaries. So a start time on 31 Aug and a finish time on 
! 2 September won't give you what you want.
!
! Note 2: Intel Fortran (at least) gives different clock rates for 
! system_clock(), depending on the type of integer fed to it. Not
! exactly what a normal person would expect.
! 
! Note 3: Using the three different standard timer routines and the
! Intel 10.04 Fortran compiler on a quad-core Intel workstation gives
! measured resolutions of:
!
!    |-----------------------------------------------|
!    |   clock function   |   resolution in seconds  |
!    |-----------------------------------------------|
!    |   cpu_time()       |    1.E-2                 |
!    |   date_and_time()  |    1.E-3                 |
!    |   system_clock()   |    1.E-4  (integer*4)    |
!    |   system_clock()   |    1.E-6  (integer*8)    |
!    |-----------------------------------------------|
! 
! Note 4: the resolution when using gfortran seems to be 1 microsecond, where
! microsecond means 9.53674316406250000E-07 = 1/1048576 seconds.
! 
! Note 5: To avoid having to put an explicit interface into every
! subprogram that calls a timer, use the timemod.f90 wrapper instead.
! Using include statements with something like 
!       include 'elapsedtime.h'
! leads to the usual out-of-sync issues. 
!
!----------------------
! R. Bramley
! Department of Computer Science
! Indiana University
!-----------------------------------
! Started: Long before 12 June 1992, when it was first time-stamped
! Modified to have Fortran 95 standard clocks, Thu Nov 20 23:22:46 2003
! Modified added date_and_time, to avoid rollovers, Mon Aug  8 13:20:43 2005
! Modified: Formatted for readability, 16 Jan 2009
! Modified: Tue 15 Sep 2009, 03:41 PM to select which timer to use, via
!       the parameter FortranVersion
! Modified: Sat 22 May 2010, 05:05 PM to set FortranVersion = 1 based
!   on resolution on coris: 1.0e-4, 1.0e-3, 1.0e-2 for the three
! Modified: Tue 22 Jun 2010, 09:26 PM to use int8 instead of int4.
!   It changes the clock rate in system_clock, at least with Intel.
! Modified: Sun 27 Jun 2010, 07:51 PM to fix the optional argument 
!   mess. Cannot assign to an optional argument, see use_dis_one
! Modified: Tue 29 Jun 2010, 06:21 PM
! Modified: Sat 20 Nov 2010, 12:11 PM for formatting
! Modified: Mon 15 Aug 2011, 07:46 PM to make arithmetic in the 4-byte integer
!   case of system_clock_version4 done in 8-byte precision
! Modified: Thu 21 Jun 2012, 02:24 PM  hardwired in which_version
! Last Modified: Thu 25 Jan 2018, 12:13 PM 
!========================================================================

function elapsedtime() result (current_time)
    implicit none
    !--------------------------------------------------------------------
    ! Normally this would use the module kindlytypes to get the kind
    ! declarations, but that makes this function less stand-alone.
    !-------------------------------------------------------------------- 
    integer, parameter :: int4  = selected_int_kind( 9)
    integer, parameter :: int8  = selected_int_kind(18)
    integer, parameter :: real4 = selected_real_kind(6,37)
    integer, parameter :: real8 = selected_real_kind(13,300)
    real(kind=real8)   :: current_time
    integer, parameter :: which_version = 8

!end function elapsedtime

    integer(kind=int4) :: cycles4, rate4, start4, finish4
    integer(kind=int8) :: cycles8, rate8, start8, finish8
    integer            :: time_array(8)
    integer, parameter :: date_and_time_version = 1
    integer, parameter :: system_clock_version4 = 4
    integer, parameter :: system_clock_version8 = 8
    integer, parameter :: cpu_time_version      = 3

!-------------------------------------------------------------------------
! Old and rarely used antiques:
!--------------------------------
!
!  RS/6000:  mclock is an integer, giving hundredths of seconds.
!        integer mclock
!        external mclock
!        elapsedtimer = dble(mclock())*0.01d0
!
!  Cray:
!       real*8 timer
!       call second(timer)
!       elapsedtimer = timer
!
!  Generic Unix: elapsed time since start of program, in seconds.
!       real etime,tarray(2)
!       external etime
!       elapsedtimer = etime(tarray)
!
!  i860 chip: dclock is real*8, returning seconds
!       real*8 dclock
!       elapsedtimer = dclock()
!
!-------------------------------------------------------------------------

    select case (which_version)

    !------------------------------------------------------------------
    ! Option 1 [time_array(4) is UTC offset and so not needed here]
    !------------------------------------------------------------------
    case (date_and_time_version)
        call date_and_time (values=time_array)
        current_time = real(time_array(8),kind=real8)*1.0d-3   &   ! number of milliseconds
                     + real(time_array(7),kind=real8)*1.0d0    &   ! number of seconds
                     + real(time_array(6),kind=real8)*6.0d1    &   ! number of minutes
                     + real(time_array(5),kind=real8)*3.6d3    &   ! number of hours
                     + real(time_array(3),kind=real8)*24*3.6d3     ! number of days

    !-------------
    ! Option 4
    !-------------
    case (system_clock_version4)
        call system_clock( count=start4, count_rate=rate4)
        ! current_time = real(start4, kind=real4)/real(rate4, kind=real4)
        current_time = real(start4, kind=real8)/real(rate4, kind=real8)


    !-------------
    ! Option 8
    !-------------
    case (system_clock_version8)
        call system_clock( count=start8, count_rate=rate8)
        current_time = real(start8, kind=real8)/real(rate8, kind=real8)

    !-------------
    ! Option 3
    !-------------
    case (cpu_time_version)
        call cpu_time(current_time)

    !---------------
    ! Non-option
    !---------------
    case default
        write(*,*) 'Dude (or dudess) read the code. gawt default case in timer'
        call system_clock( count=start8, count_rate=rate8)
        current_time = real(start8, kind=real8)/real(rate8, kind=real8)

    end select

    return

end function elapsedtime

