%--------------------------------------------------------------------------------
% 
% This Matlab script will slurp in the values for the temperatures on a plate
% discretized onto a (N+2) x (N+2) mesh. The outermost edges are all assumed
% to be boundary conditions, and this script can either plot just the  boundary
% conditions, or the full plate. If the full plate is shown, the boundary
% conditions are also displayed but using different colors.
%
% If you want to just see the boundary conditions, set
%       showBConly = true;
%       showall    = false; 
% If you want the full plate,
%       showBConly = false;
%       showall    = true; 
% Setting both to false will plot ... nothing, what did you expect?
%
% As of 16 Feb 2017, have added a mesh plot so that the solution can be compared
% to one generated from the Matlab discretizer.
%
% The input file should have two integers and a double on each row. The integers
% give the (i,j) coordinates of the corresponding value.
%
%----------------------------------
% Last Modified: Tue 23 Jan 2018, 08:18 AM
%--------------------------------------------------------------------------------

% Control parameters for what to show.
showBConly = false;
showall    = true; 
showmesh   = true;

T = load('plate');
i    = T(:,1);
j    = T(:,2);
vals = T(:,3);
N = max(i)-1;
clear T

% Can plot the whole damned thing :
if showall 
    figure
    plot3(i, j, vals, 'b+')
    xlabel('x')
    ylabel('y')
    zlabel('Temperature')
    title('Plate Temperature Values')
end

if (showBConly || showall)
    % First check the boundary conditions alone
    If = find(i == 0);
    Il = find(i == max(i));

    Jf = find(j == 0);
    Jl = find(j == max(j));

    if showBConly
        figure
        xlabel('Bottom')
        ylabel('Left-hand size')
        zlabel('Temperature')
        title(['Boundary nodes of plate: N = ' num2str(N)])
    else
        hold on
    end
    plot3(i(If), j(If), vals(If), 'c+', ...
          i(Il), j(Il), vals(Il), 'r+', ...
          i(Jf), j(Jf), vals(Jf), 'g*', ...
          i(Jl), j(Jl), vals(Jl), 'mo')
    hold off

end

if showmesh
    % Need to turn the 1D array of vals into a 2D array G for the mesh plot. 
    % Plus need to keep in mind that Matlab requires array indices to start
    % from 1.
    [X, Y] = meshgrid(0:1:N+1, 0:1:N+1);
    G = zeros(N+2, N+2);
    for iy = 0:N+1
        for ix = 0:N+1
            G(ix+1, iy+1) = vals(ix+1 + iy*(N+2));
        end
    end
    figure
    mesh(X, Y, G)
    title('Computed Solution u(x,y)');

end
